/*************************************************************
 *                 bbomberman                                *
 *  (c) 2004 by Bernhard Eder (bb-generation)                *
 *  http://bbgen.net/bomberman for updates                   *
 *  Webmaster@bbgen.net for bug reports and other stuff      *
 * --------------------------------------------------------- *
 * This program is free software; you can modify it under    *
 * the terms of the GNU General Public License as published  *
 * by the Free Software Foundation, located in LICENSE       *
 *                                                           *
 *************************************************************/

#ifndef SERVER_REMOTE_H
#define SERVER_REMOTE_H

/* cl_server_remote inherits from cl_server & uses cl_client */
#include "cl_server.h"

#include <string>

/* SDL_net is used for portable networking */
#include <SDL/SDL_net.h>


class cl_bomb_remote : public cl_bomb
{
  public:
    cl_bomb_remote(cl_bomberman& bomb_owner, const int x, const int y, const long long creation_tick) :
      cl_bomb(bomb_owner, x, y, creation_tick)
    { }
};

class cl_bomberman_remote : public cl_bomberman
{
  public:
    cl_bomberman_remote(cl_server& server, const int id, const std::string& name, const bool alive, const int x, const int y, const bb_direction direction, const int bombs_max, const int expforce, cl_client* client = NULL) :
      cl_bomberman(server, id, name, alive, x, y, direction, bombs_max, expforce, client)
    { }
};

class cl_server_remote : public cl_server
{
  public:
    /* FIXME: unite with definition in cl_server_local */
    typedef  std::vector<std::string>   bomberman_names_type;

    /* default port for BBomberman network connections */
    /* FIXME: unite with definition in cl_server_net */
    static const unsigned int  BB_NET_PORT_DEF = 34938;

    cl_server_remote(const bomberman_names_type& bomberman_names, const std::string& hostname, const Uint16 port = BB_NET_PORT_DEF);
    ~cl_server_remote();

    void       explode_field(const int x, const int y, const cl_bomberman* causer);
    bool       is_delaying() const { return true; }
    bool       is_initialized() const;
    void       tick();

  private:
    static const int  BB_NET_BUF_LEN = 256;
    //static const int  BB_NET_DELAY   = 75;
    static const int  BB_NET_DELAY   = 45;

    IPaddress  _M_host_address;
    TCPsocket  _M_socket;
    SDLNet_SocketSet  _M_socket_set;
    bool       _M_connected;
    char       _M_buffer[BB_NET_BUF_LEN];
    char*      _M_buffer_insert_ptr;
    cl_proto_queue  _M_queue;
    bool       _M_initialized;

    void       _M_pass_proto_msgs();
    void       _M_process_net_events();
    void       _M_process_server_msgs();
};

#endif
