/*************************************************************
 *                 bbomberman                                *
 *  (c) 2004 by Bernhard Eder (bb-generation)                *
 *  http://bbgen.net/bomberman for updates                   *
 *  Webmaster@bbgen.net for bug reports and other stuff      *
 * --------------------------------------------------------- *
 * This program is free software; you can modify it under    *
 * the terms of the GNU General Public License as published  *
 * by the Free Software Foundation, located in LICENSE       *
 *                                                           *
 *************************************************************/

#include "main.h"

/* used in "implementation" *hidez* */
#include <iostream>
#include "cl_init.h"
#include "cl_game.h"
#include "cl_server.h"
#include "cl_server_net.h"
#include "cl_server_remote.h"


int main(int argc, char *argv[])
{
  /* install verbose termination handler which deparses thrown exceptions */
  std::set_terminate(__gnu_cxx::__verbose_terminate_handler);

#ifdef STDERRTXT
  /* redirect stderr to "stderr.txt" */
  std::ofstream    out("stderr.txt");
  std::streambuf*  sbuf = std::cerr.rdbuf();
  std::cerr.rdbuf(out.rdbuf());
#endif

  /* create init & game object */
  cl_init  init;
  cl_game  game(init);

  /* put player names from init object together for server initialization */
  cl_server_local::bomberman_names_type  player_names;

  for (unsigned int player = 0; player < init.playercount; player++)
  {
    player_names.push_back(init.names[player]);
  }

  /* choose between local, net & remote server */
  cl_server*  server;

  std::cerr << "* main() initializing server object" << std::endl;
  switch (init.server_kind)
  {
    case BB_INIT_SERVER_LOCAL:
      server = new cl_server_local(init.fX, init.fY, player_names);
      break;
    case BB_INIT_SERVER_NET:
      server = new cl_server_net(init.fX, init.fY, player_names);
      break;
    case BB_INIT_SERVER_REMOTE:
      //server = new cl_server_remote(player_names, init->server_hostname, init->server_port);
      server = new cl_server_remote(player_names, init.server_hostname);
      break;
    default:
      server = new cl_server_local(init.fX, init.fY, player_names);
      break;
  }

  /* play the game ;) */
  game.play(*server);

  /* clean up server object */
  delete server;

#ifdef STDERRTXT
  /* reset default stderr */
  std::cerr.rdbuf(sbuf);
#endif

  return 0;
}
